package be.iminds.ilabt.jfed.experimenter_gui.ui;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Resource;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;
import org.controlsfx.glyphfont.Glyph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.util.Objects;

/**
 * Visualisation of JFedGuiConfig.ComponentInfo in a combobox
 */
public class ComponentInfoCell extends ListCell<JFedGuiConfig.ComponentInfo> {
    private final static Logger LOG = LoggerFactory.getLogger(ComponentInfoCell.class);

    @Nonnull private final Resource resource;
    @Nonnull private final JFedGuiConfig config;

    protected final HBox container = new HBox(2);

    protected final Pane fillerPane;
    protected final Label nameLabel = new Label("--init--");
    protected final Glyph availableIcon =
            GlyphUtils.createGlyph(FontAwesome.Glyph.CALENDAR, Color.GREEN);
    protected final Glyph loginIcon =
            GlyphUtils.createGlyph(FontAwesome.Glyph.TERMINAL, Color.GREEN);
    protected final Glyph sensorGlyph = GlyphUtils.createGlyph(FontAwesome.Glyph.TACHOMETER, Color.GREEN);

    public ComponentInfoCell(@Nonnull Resource resource, @Nonnull JFedGuiConfig config) {
        this.config = config;
        this.resource = resource;

        this.fillerPane = new Pane();
        HBox.setHgrow(fillerPane, Priority.ALWAYS);

        container.setPadding(new Insets(0, 5, 2, 0));
        container.getChildren().addAll(nameLabel, fillerPane, availableIcon, loginIcon, sensorGlyph);
        container.setMinWidth(USE_PREF_SIZE);
        nameLabel.setMinWidth(USE_PREF_SIZE);

        setGraphic(container);
    }


    @Override
    protected void updateItem(JFedGuiConfig.ComponentInfo item, boolean empty) {
        super.updateItem(item, empty);

        setText(null);
        setGraphic(container);
        container.setMinWidth(USE_PREF_SIZE);
        nameLabel.setMinWidth(USE_PREF_SIZE);

        if (!empty && item != null) {
            if (item.getName().trim().isEmpty())
                nameLabel.setText("-- no name --");
            else
                nameLabel.setText(item.getName());

            if (item.hasExtraStatus(JFedGuiConfig.ComponentInfo.ExtraStatusInfoType.NODE_LOGIN)) {
                loginIcon.setVisible(true);
                loginIcon.setManaged(true);
                if (Objects.equals(item.isSuccess(JFedGuiConfig.ComponentInfo.ExtraStatusInfoType.NODE_LOGIN), Boolean.TRUE))
                    loginIcon.setTextFill(Color.GREEN);
                else
                    loginIcon.setTextFill(Color.RED);
            } else {
                loginIcon.setVisible(false);
                loginIcon.setManaged(false);
            }

            if (item.hasExtraStatus(JFedGuiConfig.ComponentInfo.ExtraStatusInfoType.SENSOR)) {
                sensorGlyph.setVisible(true);
                sensorGlyph.setManaged(true);
                if (Objects.equals(item.isSuccess(JFedGuiConfig.ComponentInfo.ExtraStatusInfoType.SENSOR), Boolean.TRUE))
                    sensorGlyph.setTextFill(Color.GREEN);
                else
                    sensorGlyph.setTextFill(Color.RED);
//                    sensorGlyph.setTextFill(Color.GRAY);
            } else {
                sensorGlyph.setVisible(false);
                sensorGlyph.setManaged(false);

            }

            if (item.isAvailable())
                nameLabel.setTextFill(Color.BLACK);
            else
                nameLabel.setTextFill(Color.GRAY);
            availableIcon.setVisible(false);
            availableIcon.setManaged(false);
//            availableIcon.setVisible(true);
//            availableIcon.setManaged(true);
//            if (item.isAvailable())
//                availableIcon.setTextFill(Color.GREEN);
//            else
//                availableIcon.setTextFill(Color.GRAY);
        } else {
            nameLabel.setText(""/*"-- none selected --"*/);
            loginIcon.setVisible(false);
            loginIcon.setManaged(false);
            sensorGlyph.setVisible(false);
            sensorGlyph.setManaged(false);
            availableIcon.setVisible(false);
            availableIcon.setManaged(false);
        }
        nameLabel.setVisible(true);
        nameLabel.setManaged(true);
    }

}
